package de.mdelab.mlsdm.interpreter.debug.ui.breakpoints.view;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.ui.ISaveablePart;
import org.eclipse.ui.IWorkbenchPartConstants;
import org.eclipse.ui.part.ViewPart;

import de.mdelab.expressiondialogs.SourceViewerProvider;
import de.mdelab.expressiondialogs.figures.MlstorypatternsDiagramConstants;
import de.mdelab.mlcallactions.VariableDeclarationAction;
import de.mdelab.mlexpressions.MlexpressionsFactory;
import de.mdelab.mlexpressions.MlexpressionsPackage;
import de.mdelab.mlsdm.Activity;
import de.mdelab.mlsdm.ActivityParameter;
import de.mdelab.mlstorypatterns.MlstorypatternsUtils;
import de.mdelab.mlstorypatterns.StoryPatternObject;

public abstract class AbstractDebugView extends ViewPart implements ISaveablePart
{

	protected final Map<String, StringExpressionView>	VIEW_MAP;

	protected ISDBreakpointItem							currentItem;

	protected Map<String, ISourceViewer>				sourceViewers;

	protected final Map<String, SourceViewerProvider>	sourceViewerProviders;

	protected Activity									activity;

	private boolean										fDirty	= false;

	EClassifier											contextClassifier;

	public AbstractDebugView()
	{

		this.VIEW_MAP = new HashMap<String, StringExpressionView>();

		this.VIEW_MAP.put(MlexpressionsPackage.Literals.ML_STRING_EXPRESSION.getName(), new StringExpressionView());

		/*
		 * Create the map of registered source viewers. This map maps the
		 * expression languages to their associated source viewers.
		 */
		this.sourceViewerProviders = new HashMap<String, SourceViewerProvider>();
		final IConfigurationElement[] configurationElements = Platform.getExtensionRegistry().getConfigurationElementsFor(
				MlstorypatternsDiagramConstants.EXPRESSION_SOURCE_VIEWER_EXTENSION_POINT_ID);

		for (final IConfigurationElement configurationElement : configurationElements)
		{
			final String s = configurationElement.getAttribute(MlstorypatternsDiagramConstants.EXPRESSION_LANGUAGE);

			if ((s != null) && !"".equals(s))
			{
				try
				{
					this.sourceViewerProviders.put(s, (SourceViewerProvider) configurationElement
							.createExecutableExtension(MlstorypatternsDiagramConstants.SOURCE_VIEWER_PROVIDER));
				}
				catch (final CoreException e)
				{
					// Skip it but show error message
					e.printStackTrace();
				}
			}
		}

		/*
		 * Add a default source viewer for all expression languages that do not
		 * have one.
		 */
		for (final String expressionLanguage : MlstorypatternsUtils.getAvailableExpressionLanguages())
		{
			if (!this.sourceViewerProviders.containsKey(expressionLanguage))
			{
				this.sourceViewerProviders.put(expressionLanguage, new SourceViewerProvider());
			}
		}
	}

	protected abstract void addOrResetStringExpressionEditor(String expressionLanguage, String expression);

	public void setContextClassifier(final EClassifier contextClassifier)
	{
		this.contextClassifier = contextClassifier;
	}

	public interface LanguageSelectionWidget
	{
		public abstract void addLanguageSelectedListener(LanguageSelectedListener listener);

		public abstract void removeLanguageSelectedListener(LanguageSelectedListener listener);

		public abstract void setSelectedLanguage(String language);

		public abstract void addLanguage(String language);

		public abstract String getSelectedLanguage();
	}

	class StringExpressionView
	{
		public void adaptInputElements()
		{
			AbstractDebugView.this.addOrResetStringExpressionEditor(AbstractDebugView.this.currentItem.getExpressionLanguage(),
					AbstractDebugView.this.currentItem.getExpression());
		}

		public EObject create()
		{
			return MlexpressionsFactory.eINSTANCE.createMLStringExpression();
		}

		public void processCurrentInput()
		{

			final SourceViewerProvider svp = AbstractDebugView.this.sourceViewerProviders.get(AbstractDebugView.this.currentItem
					.getExpressionLanguage());
			if ((AbstractDebugView.this.sourceViewerProviders.size() == 0) || (svp == null))
			{
				return;
			}
		}
	}

	public class LanguageSelectedEvent extends java.util.EventObject
	{

		public String	language;

		public LanguageSelectedEvent(final Object source, final String language)
		{
			super(source);
			this.language = language;
		}
	}

	public interface LanguageSelectedListener extends java.util.EventListener
	{
		public void languageSelected(LanguageSelectedEvent event);
	}

	protected Map<String, EClassifier> getContextInformation()
	{
		final Map<String, EClassifier> contextInfos = new HashMap<String, EClassifier>();

		if (this.activity != null)
		{
			/*
			 * THIS and parameters
			 */
			if (this.activity != null)
			{
				// EClass thisClass =
				// activity.getSpecification().getEContainingClass();

				// if (thisClass != null)
				// {
				// contextInfos.put("this", thisClass);
				// }

				for (final ActivityParameter parameter : this.activity.getParameters())
				{
					if ((parameter.getName() != null) && !"".equals(parameter) && (parameter.getType() != null))
					{
						contextInfos.put(parameter.getName(), parameter.getType());
					}
				}
			}

			/*
			 * Variables created in story patterns and
			 * VariableDeclarationActions
			 */
			final TreeIterator<EObject> it = this.activity.eAllContents();

			while (it.hasNext())
			{
				final EObject eObject = it.next();

				if (eObject instanceof StoryPatternObject)
				{
					final StoryPatternObject spo = (StoryPatternObject) eObject;

					if ((spo.getName() != null) && !"".equals(spo.getName()) && (spo.getType() != null))
					{
						contextInfos.put(spo.getName(), spo.getType());
					}
				}
				else if (eObject instanceof VariableDeclarationAction)
				{
					final VariableDeclarationAction vda = (VariableDeclarationAction) eObject;

					if ((vda.getName() != null) && !"".equals(vda.getName()) && (vda.getType() != null))
					{
						contextInfos.put(vda.getName(), vda.getType());
					}
				}
			}
		}
		return contextInfos;
	}

	protected void setDirty(final boolean dirty)
	{
		if (this.fDirty != dirty)
		{
			this.fDirty = dirty;
			this.firePropertyChange(IWorkbenchPartConstants.PROP_DIRTY);
		}
	}

	@Override
	public void doSave(final IProgressMonitor monitor)
	{

		if (this.fDirty)
		{
			System.out.println("Saving BP");
			this.currentItem.saveChanges();
			this.setDirty(false);
		}
	}

	@Override
	public void doSaveAs()
	{

	}

	@Override
	public boolean isDirty()
	{
		return this.fDirty;
	}

	@Override
	public boolean isSaveAsAllowed()
	{
		return false;
	}

	@Override
	public boolean isSaveOnCloseNeeded()
	{
		return true;
	}
}
